/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_DIRECTORY_H
#define __TOOLBOX_DIRECTORY_H


/**
 * The file has its read only flag set.
 */
#define DU_FREADDONLY 0x01

/**
 * The file has its hidden flag set.
 */
#define DU_FHIDDEN    0x02

/**
 * The file has its system flag set.
 */
#define DU_FSYSTEM    0x04

/**
 * The file is a directory.
 */
#define DU_FDIRECTORY 0x10

/**
 * The file has its archive flag set.
 */
#define DU_FARCHIVE   0x20

/**
 * Specifies all flags. This value is useful when
 * using the Directory::ReadFrom methods.
 */
#define DU_FALLFLAGS  0x3F

/**
 * Specifies no flags. This value is useful when
 * using the Directory::ReadFrom methods.
 */
#define DU_FNOFLAGS   0x00

/**
 * Indicates, that the value is invalid.
 */
#define DU_INVALID -1


namespace toolbox
{
	/**
	 * The Directory class stores information about
	 * a directory, its sub-directories and all files in
	 * the directories. It can be used to get all information
	 * at once, without having to handle the recursive calls.
	 * The files and directories are stored in a flat structure.
	 * @author Thomas Jacob
	 */
	class Directory
	{
		private:

			/**
			 * The entries of the read directory.
			 */
			ContainingList<DirectoryEntry> * Entries;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The main directory read recursion. Gets all matching
			 * files and/or directories and recurses, if desired,
			 * and fills the directory object with the entries.
			 * @param RootWithFilter The directory root to read from.
			 * @param requiredAttributes The attributes the files
			 * and/or directories have to have, to be added to the
			 * directory object.
			 * @param forbiddenAttributes The attributes the files
			 * and/or directories must not have, to be added to the
			 * directory object.
			 * @param recurse If the method shall recurse
			 * into sub-directories.
			 */
			void ReadFromInternal(Path * RootWithFilter,
				int requiredAttributes, int forbiddenAttributes, bool recurse);

		public:

			/**
			 * Creates a new directory.
			 */
			Directory();

			/**
			 * Destroys the directory.
			 */
			~Directory();

			/**
			 * Appends an entry to the directory object.
			 * @param Entry The entry to be appended.
			 */
			inline void AppendEntry(DirectoryEntry * Entry);

			/**
			 * Removes and frees all entries in the directory object.
			 */
			inline void DeleteAll();

			/**
			 * Removes and frees a directory entry.
			 * @param nr The position of the entry.
			 * @return If the removal was successful.
			 */
			inline bool DeleteEntry(long nr);

			/**
			 * Removes and frees a directory entry.
			 * @param Entry The entry to be removed.
			 * @return If the removal was successful.
			 */
			inline bool DeleteEntry(DirectoryEntry * Entry);

			/**
			 * Removes and frees the first directory entry.
			 * @return If the removal was successful.
			 */
			inline bool DeleteFirstEntry();

			/**
			 * Removes and frees the last directory entry.
			 * @return If the removal was successful.
			 */
			inline bool DeleteLastEntry();

			/**
			 * Returns the directory entry at a given position.
			 * @param nr The position of the entry.
			 * @return The entry at the position.
			 */
			inline DirectoryEntry * GetEntry(long nr);

			/**
			 * Returns the number of directory entries.
			 * @return The number of directory entries.
			 */
			inline int GetEntryCount();

			/**
			 * Returns the first directory entry.
			 * @return The first directory entry.
			 */
			inline DirectoryEntry * GetFirstEntry();

			/**
			 * Returns the last directory entry.
			 * @return The last directory entry.
			 */
			inline DirectoryEntry * GetLastEntry();

			/**
			 * Inserts an directory entry before a given position.
			 * All elements at and after the position
			 * are moved by one.
			 * @param Entry The entry to be inserted.
			 * @param nr The position to insert before.
			 */
			inline void InsertEntry(DirectoryEntry * Entry, long nr);

			/**
			 * Returns, if there are no entries in the directory.
			 */
			inline bool IsEmpty();

			/**
			 * Prepends an entry to the directory object.
			 * @param Entry The entry to be prepended.
			 */
			inline void PrependEntry(DirectoryEntry * Entry);

			/**
			 * Adds all files and/or directories from a given root
			 * to the directory object.
			 * @param rootWithFilter The root to start from,
			 * appended by the filter for all files in the
			 * directory or sub-directories. The sub-directories
			 * themselves are not filtered by the filter.
			 * @param requiredAttributes The attributes the files
			 * and/or directories have to have, to be added to the
			 * directory object.
			 * @param forbiddenAttributes The attributes the files
			 * and/or directories must not have, to be added to the
			 * directory object.
			 * @param recurse If the method shall recurse
			 * into sub-directories.
			 */
			void ReadFrom(const char * rootWithFilter,
				int requiredAttributes, int forbiddenAttributes, bool recurse);

			/**
			 * Adds all files and/or directories from a given root
			 * to the directory object.
			 * @param root The directory root to read from.
			 * @param filter The filter for all files in the
			 * directory or sub-directories. The sub-directories
			 * themselves are not filtered by the filter.
			 * @param requiredAttributes The attributes the files
			 * and/or directories have to have, to be added to the
			 * directory object.
			 * @param forbiddenAttributes The attributes the files
			 * and/or directories must not have, to be added to the
			 * directory object.
			 * @param recurse If the method shall recurse
			 * into sub-directories.
			 */
			void ReadFrom(const char * root, const char * filter,
				int requiredAttributes, int forbiddenAttributes, bool recurse);

			/**
			 * Adds all files and/or directories from a given root
			 * to the directory object.
			 * @param RootWithFilter The root to start from,
			 * appended by the filter for all files in the
			 * directory or sub-directories. The sub-directories
			 * themselves are not filtered by the filter.
			 * @param requiredAttributes The attributes the files
			 * and/or directories have to have, to be added to the
			 * directory object.
			 * @param forbiddenAttributes The attributes the files
			 * and/or directories must not have, to be added to the
			 * directory object.
			 * @param recurse If the method shall recurse
			 * into sub-directories.
			 */
			void ReadFrom(Path * RootWithFilter,
				int requiredAttributes, int forbiddenAttributes, bool recurse);

			/**
			 * Adds all files and/or directories from a given root
			 * to the directory object.
			 * @param Root The directory root to read from.
			 * @param filter The filter for all files in the
			 * directory or sub-directories. The sub-directories
			 * themselves are not filtered by the filter.
			 * @param requiredAttributes The attributes the files
			 * and/or directories have to have, to be added to the
			 * directory object.
			 * @param forbiddenAttributes The attributes the files
			 * and/or directories must not have, to be added to the
			 * directory object.
			 * @param recurse If the method shall recurse
			 * into sub-directories.
			 */
			void ReadFrom(Path * Root, const char * filter,
				int requiredAttributes, int forbiddenAttributes, bool recurse);

			/**
			 * Removes a directory entry at a given position.
			 * The entry is not freed, but returned.
			 * @param nr The position of the entry.
			 * @return The directory entry.
			 */
			inline DirectoryEntry * UnlinkEntry(long nr);

			/**
			 * Removes a given directory entry from the array.
			 * The entry is not freed, but returned.
			 * @param Entry The entry to be removed.
			 * @return The directory entry.
			 */
			inline DirectoryEntry * UnlinkEntry(DirectoryEntry * Entry);

			/**
			 * Removes the first directory entry from the array.
			 * The entry is not freed, but returned.
			 * @return The directory entry.
			 */
			inline DirectoryEntry * UnlinkFirstEntry();

			/**
			 * Removes the last directory entry from the array.
			 * The entry is not freed, but returned.
			 * @return The directory entry.
			 */
			inline DirectoryEntry * UnlinkLastEntry();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A DirectoryEntry in the Directory object.
	 * It represents the Path and information
	 * of a file or directory.
	 * @author Thomas Jacob
	 */
	class DirectoryEntry : public ContainingListItem<DirectoryEntry>
	{
		friend Directory;

		private:

			/**
			 * The last access time.
			 */
			struct tm AccessTime;

			/**
			 * The creation time.
			 */
			struct tm CreateTime;

			/**
			 * If the file or directory exists.
			 */
			bool ExistsFlag;

			/**
			 * The file or directory flags.
			 */
			long Flags;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The last modification time.
			 */
			struct tm ModifyTime;

			/**
			 * The complete path to the file or directory.
			 */
			class Path * Path;

			/**
			 * The size of the file.
			 */
			unsigned long Size;

			/**
			 * Creates a new directory entry.
			 */
			DirectoryEntry();

			/**
			 * Invalidates all directory entry's data.
			 * This is done when the entry is refreshed,
			 * and the file or directory cannot be found.
			 */
			void Invalidate();

			/**
			 * Invalidates a time attribute of the entry's data.
			 * This is done when the entry is refreshed,
			 * and the file or directory cannot be found.
			 * @param time A pointer to the attribute
			 * to be invalidated.
			 */
			static void InvalidateTime(struct tm * time);

		public:

			/**
			 * Creates a new directory entry.
			 * @param Path A path to the file or directory.
			 * This path is copied, so the memory is not used
			 * anymore after creation.
			 */
			DirectoryEntry(class Path * Path);

			/**
			 * Creates a new directory entry.
			 * @param path A path to the file or directory.
			 * This path is copied, so the memory is not used
			 * anymore after creation.
			 */
			DirectoryEntry(const char * path);

			/**
			 * Destroys the directory entry.
			 */
			~DirectoryEntry();

			/**
			 * Creates a new instance of the directory entry
			 * with the same content.
			 * @return A clone of the object.
			 * @exception EOutOfMemory The clone could not be allocated.
			 */
			DirectoryEntry * Clone();

			/**
			 * Returns, if the file or directory, the
			 * path points to, exists.
			 * @return If the file or directory exists.
			 */
			inline bool Exists();

			/**
			 * Returns the last access time.
			 * @return The last access time.
			 */
			long GetAccessTime();

			/**
			 * Returns the seconds of the last access time.
			 * @return The seconds of the last access time.
			 */
			inline int GetAccessTimeSecond();

			/**
			 * Returns the minutes of the last access time.
			 * @return The minutes of the last access time.
			 */
			inline int GetAccessTimeMinute();

			/**
			 * Returns the hours of the last access time.
			 * @return The hours of the last access time.
			 */
			inline int GetAccessTimeHour();

			/**
			 * Returns the days of the last access time.
			 * @return The days of the last access time.
			 */
			inline int GetAccessTimeDay();

			/**
			 * Returns the months of the last access time.
			 * @return The months of the last access time.
			 */
			inline int GetAccessTimeMonth();

			/**
			 * Returns the years of the last access time.
			 * @return The years of the last access time.
			 */
			inline int GetAccessTimeYear();

			/**
			 * Returns the last creation time.
			 * @return The last creation time.
			 */
			long GetCreateTime();

			/**
			 * Returns the seconds of the last creation time.
			 * @return The seconds of the last creation time.
			 */
			inline int GetCreateTimeSecond();

			/**
			 * Returns the minutes of the last creation time.
			 * @return The minutes of the last creation time.
			 */
			inline int GetCreateTimeMinute();

			/**
			 * Returns the hours of the last creation time.
			 * @return The hours of the last creation time.
			 */
			inline int GetCreateTimeHour();

			/**
			 * Returns the days of the last creation time.
			 * @return The days of the last creation time.
			 */
			inline int GetCreateTimeDay();

			/**
			 * Returns the months of the last creation time.
			 * @return The months of the last creation time.
			 */
			inline int GetCreateTimeMonth();

			/**
			 * Returns the years of the last creation time.
			 * @return The years of the last creation time.
			 */
			inline int GetCreateTimeYear();

			/**
			 * Returns the flags of the file or directory.
			 * @return The flags of the file or directory.
			 */
			inline long GetFlags();

			/**
			 * Returns the last modification time.
			 * @return The last modification time.
			 */
			long GetModifyTime();

			/**
			 * Returns the seconds or the last modification time.
			 * @return The seconds or the last modification time.
			 */
			inline int GetModifyTimeSecond();

			/**
			 * Returns the minutes or the last modification time.
			 * @return The minutes or the last modification time.
			 */
			inline int GetModifyTimeMinute();

			/**
			 * Returns the hours or the last modification time.
			 * @return The hours or the last modification time.
			 */
			inline int GetModifyTimeHour();

			/**
			 * Returns the days or the last modification time.
			 * @return The days or the last modification time.
			 */
			inline int GetModifyTimeDay();

			/**
			 * Returns the months or the last modification time.
			 * @return The months or the last modification time.
			 */
			inline int GetModifyTimeMonth();

			/**
			 * Returns the years or the last modification time.
			 * @return The years or the last modification time.
			 */
			inline int GetModifyTimeYear();

			/**
			 * Returns the next directory entry.
			 * @return The next directory entry.
			 */
			inline class DirectoryEntry * GetNextEntry();

			/**
			 * Returns the path of the file or directory.
			 * @return The path of the file or directory.
			 */
			inline class Path * GetPath();

			/**
			 * Returns the previous directory entry.
			 * @return The previous directory entry.
			 */
			inline class DirectoryEntry * GetPrevEntry();

			/**
			 * Returns the file's size.
			 * @return The file's size.
			 */
			inline unsigned long GetSize();

			/**
			 * Refreshed the file or directory's attributes
			 * from the disk.
			 */
			void Refresh();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A path describes the position of a file or directory,
	 * absolutely or relatively. This description consists
	 * of a root, the directories and/or file elements.
	 * @author Thomas Jacob
	 */
	class Path
	{
		private:

			/**
			 * The directories and/or file elements.
			 */
			ContainingList<PathElement> * Elements;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The root of the file or directory.
			 * This is for example C:\.
			 */
			char * Root;

		public:

			/**
			 * Creates an empty path.
			 */
			Path();

			/**
			 * Creates a path from a string template.
			 * @param path The path as a string. This string
			 * is no longer used after the construction.
			 */
			Path(const char * path);

			/**
			 * Destroys the path.
			 */
			~Path();

			/**
			 * Appends a path element to the path.
			 * @param elementName A string for a new path
			 * element be appended.
			 */
			inline void AppendElement(const char * elementName);

			/**
			 * Appends a path element to the path.
			 * @param Element The element to be appended.
			 * This element is not copied, but directly used.
			 */
			inline void AppendElement(PathElement * Element);

			/**
			 * Appends another path's elements to the path.
			 * @param path A string for a new path elements
			 * to be appended.
			 */
			void AppendPath(const char * path);

			/**
			 * Appends another path's elements to the path.
			 * @param Path The path to be appended. The elements
			 * are cloned, so the path is no longer used after
			 * it has been appended.
			 */
			void AppendPath(class Path * Path);

			/**
			 * Creates a new instance of the path with
			 * clones of the path elements.
			 * @return A clone of the object.
			 * @exception EOutOfMemory The clone could not be allocated.
			 */
			Path * Clone();

			/**
			 * Returns whether a given path is a part of this path.
			 * That is, the roots are the same, and all the given path's
			 * elements match the first elements of this path.
			 * @param path The path that must be the prefix of this path.
			 * @return Whether a given path is a part of this path.
			 */
			bool Contains(const char * path);

			/**
			 * Returns whether a given path is a part of this path.
			 * That is, the roots are the same, and all the given path's
			 * elements match the first elements of this path.
			 * @param Path The path that must be the prefix of this path.
			 * @return Whether a given path is a part of this path.
			 */
			bool Contains(class Path * Path);

			/**
			 * Creates a new instance of the path
			 * that points to the application (EXE) file's container folder.
			 * @return The application folder path.
			 * @exception EOutOfMemory The object could not be allocated.
			 */
			static Path * CreateApplicationFolderPath();

			/**
			 * Creates a new instance of the path
			 * that points to the application (EXE) file.
			 * @return The application path.
			 * @exception EOutOfMemory The object could not be allocated.
			 */
			static Path * CreateApplicationPath();

			/**
			 * Creates a new relative path from this path to another one.
			 * If both paths are relative, or both are absolute with the same root,
			 * the method creates a relative path from this path to the other one.<br>
			 * If both paths are absolute with a different root,
			 * or if this path is relative and the other one is absolute,
			 * the method creates a clone of the other (absolute) path.<br>
			 * If this path is absolute and the other path is relative,
			 * the method returns NULL, because a path cannot be built.<br>
			 *
			 * Examples:<br>
			 * This is foo, other path is bar.dat, result is ..\bar.dat<br>
			 * This is C:\foo, other path is C:\bar.dat, result is ..\bar.dat<br>
			 * This is foo, other path is C:\bar.dat, result is C:\bar.dat<br>
			 * This is C:\foo, other path is D:\bar.dat, result is D:\bar.dat<br>
			 * This is C:\foo, other path is bar.dat, result is NULL
			 *
			 * @param path The other path.
			 * @return A new Path object or NULL.
			 * @warning This method allocates memory and expects,
			 * that it is freed by the caller.
			 */
			Path * CreateRelativePathTo(const char * path);

			/**
			 * Creates a new relative path from this path to another one.
			 * If both paths are relative, or both are absolute with the same root,
			 * the method creates a relative path from this path to the other one.<br>
			 * If both paths are absolute with a different root,
			 * or if this path is relative and the other one is absolute,
			 * the method creates a clone of the other (absolute) path.<br>
			 * If this path is absolute and the other path is relative,
			 * the method returns NULL, because a path cannot be built.<br>
			 *
			 * Examples:<br>
			 * This is foo, other path is bar.dat, result is ..\bar.dat<br>
			 * This is C:\foo, other path is C:\bar.dat, result is ..\bar.dat<br>
			 * This is foo, other path is C:\bar.dat, result is C:\bar.dat<br>
			 * This is C:\foo, other path is D:\bar.dat, result is D:\bar.dat<br>
			 * This is C:\foo, other path is bar.dat, result is NULL
			 *
			 * @param Path The other path.
			 * @return A new Path object or NULL.
			 * @warning This method allocates memory and expects,
			 * that it is freed by the caller.
			 */
			class Path * CreateRelativePathTo(class Path * Path);

			/**
			 * Creates the path that results when navigation from this path
			 * to another path. If the other path is relative, it is appended
			 * to this path. If it is absolute, a clone of that path is returned.<br>
			 * The method always creates a new path.<br>
			 *
			 * @param path The other path.
			 * @return The target path.
			 * @warning This method allocates memory and expects,
			 * that it is freed by the caller.
			 */
			class Path * CreateTargetPath(const char * path);

			/**
			 * Creates the path that results when navigation from this path
			 * to another path. If the other path is relative, it is appended
			 * to this path. If it is absolute, a clone of that path is returned.<br>
			 * The method always creates a new path.<br>
			 *
			 * @param Path The other path.
			 * @return The target path.
			 * @warning This method allocates memory and expects,
			 * that it is freed by the caller.
			 */
			class Path * CreateTargetPath(class Path * Path);

			/**
			 * Deletes all path elements from the path.
			 * All elements are removed from the path and freed.
			 */
			inline void DeleteAll();

			/**
			 * Deletes a path element at a given position.
			 * The element is removed from the path and freed.
			 * @param nr The position of the element.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteElement(long nr);

			/**
			 * Deletes a given path element from the path.
			 * The element is searched, removed from the path and freed.
			 * @param Element The element to be deleted.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteElement(PathElement * Element);

			/**
			 * Deletes the first path element from the path.
			 * The element is removed from the path and freed.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteFirstElement();

			/**
			 * Deletes the last path element from the path.
			 * The element is removed from the path and freed.
			 * @return If the deletion was successful.
			 */
			inline bool DeleteLastElement();

			/**
			 * Returns, if the file or directory, the
			 * path points to, exists.
			 * @return If the file or directory exists.
			 */
			bool Exists();

			/**
			 * Returns the path element at a given position.
			 * @param nr The position of the element.
			 * @return The element at the position.
			 */
			inline PathElement * GetElement(long nr);

			/**
			 * Returns the number of path elements.
			 * @return The number of path elements.
			 */
			inline int GetElementCount();

			/**
			 * Returns the first path element.
			 * @return The first path element.
			 */
			inline PathElement * GetFirstElement();

			/**
			 * Returns the last path element.
			 * @return The last path element.
			 */
			inline PathElement * GetLastElement();

			/**
			 * Returns the root of the path (e.g. "C:\").
			 * If the path is relative, this is an empty string.
			 * @return The root of the path.
			 */
			inline char * GetRoot();

			/**
			 * Inserts a path element before a given position.
			 * @param elementName A string for a new path
			 * element be inserted.
			 * @param nr The position to insert before.
			 */
			inline void InsertElement(const char * elementName, long nr);

			/**
			 * Inserts a path element before a given position.
			 * @param Element The element to be inserted.
			 * This element is not copied, but directly used.
			 * @param nr The position to insert before.
			 */
			inline void InsertElement(PathElement * Element, long nr);

			/**
			 * Returns, if the Root is not empty, so if the Path is absolute.
			 * @return If the Path is absolute.
			 */
			inline bool IsAbsolute();

			/**
			 * Returns, if the path elements are empty.
			 * @return If the path elements are empty.
			 */
			inline bool IsEmpty();

			/**
			 * Returns, if the Root is empty, so if the Path is relative.
			 * @return If the Path is relative.
			 */
			inline bool IsRelative();

			/**
			 * Makes a directory with the name
			 * the path points to.
			 */
			bool MakeDirectory();

			/**
			 * Prepends a path element to the path.
			 * @param elementName A string for a new path
			 * element be prepended.
			 */
			inline void PrependElement(const char * elementName);

			/**
			 * Prepends a path element to the path.
			 * @param Element The element to be prepended.
			 * This element is not copied, but directly used.
			 */
			inline void PrependElement(PathElement * Element);

			/**
			 * Prepends another path's elements to the path.
			 * @param path A string for a new path elements
			 * to be prepended.
			 */
			void PrependPath(const char * path);

			/**
			 * Prepends another path's elements to the path.
			 * @param Path The path to be prepended. The elements
			 * are cloned, so the path is no longer used after
			 * it has been prepended.
			 */
			void PrependPath(class Path * Path);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets the root for the path. This is for example C:\.
			 * After the root has been set, it is no longer used.
			 * @param root The new root.
			 */
			void SetRoot(const char * root);

			/**
			 * Creates a string representing the complete path.
			 * It is not trailed by a back slash, if not
			 * required by the root.
			 * @return The path as string.
			 * @warning This method creates memory and expects
			 * that it is freed by the caller.
			 */
			inline char * ToString();

			/**
			 * Creates a string representing the complete path.
			 * @param trailingSlash If it shall be trailed by
			 * a back slash, if not required by the root.
			 * @return The path as string.
			 * @warning This method creates memory and expects
			 * that it is freed by the caller.
			 */
			char * ToString(bool trailingSlash);

			/**
			 * Removes an element at a given position.
			 * The element is not freed, but returned.
			 * @param nr The position of the element.
			 * @return The element.
			 */
			inline PathElement * UnlinkElement(long nr);

			/**
			 * Removes an given element from the path.
			 * The element is not freed, but returned.
			 * @param Element The element to be removed.
			 * @return The element.
			 */
			inline PathElement * UnlinkElement(PathElement * Element);

			/**
			 * Removes the first element from the path.
			 * The element is not freed, but returned.
			 * @return The element.
			 */
			inline PathElement * UnlinkFirstElement();

			/**
			 * Removes the last element from the path.
			 * The element is not freed, but returned.
			 * @return The element.
			 */
			inline PathElement * UnlinkLastElement();
	};


	/**
	 * A path element is a directory or file name,
	 * a part of a path.
	 * @author Thomas Jacob
	 */
	class PathElement : public ContainingListItem<PathElement>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The content of the path element itself.
			 */
			char * Name;

		public:

			/**
			 * Creates an empty path element.
			 */
			PathElement();

			/**
			 * Creates a new path element from a string template.
			 * @param Name The path element as a string. This string
			 * is no longer used after the construction.
			 */
			PathElement(const char * Name);

			/**
			 * Destroys the path element.
			 */
			~PathElement();

			/**
			 * Changes the extension of the path element.
			 * @param newExtension The new extension.
			 * @return If the extension was successfully changed.
			 */
			bool ChangeExtension(const char * newExtension);

			/**
			 * Creates a new instance of the path element
			 * with the same content.
			 * @return A clone of the object.
			 * @exception EOutOfMemory The clone could not be allocated.
			 */
			PathElement * Clone();

			/**
			 * Returns the content of the path element.
			 * @return The content of the path element.
			 */
			inline char * GetName();

			/**
			 * Returns the next path element.
			 * @return The next path element.
			 */
			inline class PathElement * GetNextElement();

			/**
			 * Returns the previous path element.
			 * @return The previous path element.
			 */
			inline class PathElement * GetPrevElement();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets the content of the path element.
			 * After the content has been set, it is no longer used.
			 * @param name The new content.
			 *
			 */
			void SetName(const char * name);
	};
}


#endif
