/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class Exception ---

Exception::Exception(const char * className, const char * message, const char * source, long lineNumber)
{
	ClassName = className;
	LineNumber = lineNumber;
	Message = NULL;
	Source = NULL;

	SetMessage(message);
	SetSource(source);

	if (source != NULL && lineNumber != 0)
		Logger::Get()->Printf(source, lineNumber, LOG_EXCEPTION,
			"%s.\nMessage: %s", className, message);
}

Exception::~Exception()
{
	ASSERTION_THIS;

	if (Message) {delete Message; Message = NULL;}
	if (Source) {delete Source; Source = NULL;}
}

void Exception::SetMessage(const char * message)
{
	ASSERTION_THIS;

	if ((Message = new char[strlen(message)+1]) == NULL)
		throw EOutOfMemory::Get();
	else
		strcpy(Message, message);
}

void Exception::SetSource(const char * source)
{
	ASSERTION_THIS;

	const char * fileNamePtr = strrchr(source, '\\');
	if (fileNamePtr != NULL)
		source = &fileNamePtr[1];

	if ((Source = new char[strlen(source)+1]) == NULL)
		throw EOutOfMemory::Get();
	else
		strcpy(Source, source);
}


// --- Class EAutoBreaking

EAutoBreaking::EAutoBreaking(const char * className, const char * message, const char * source, long lineNumber)
	: Exception(className, message, source, lineNumber)
{
	#if defined(_DEBUG) && defined(_WINDOWS)
		DebugBreak();
	#endif
}


// --- Class EAssertionFailed ---

EAssertionFailed::EAssertionFailed(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("EAssertionFailed", message, source, lineNumber)
{
}

void EAssertionFailed::Assertion(bool assertion, const char * message,
	const char * source, long lineNumber)
{
	if (!assertion)
		throw new EAssertionFailed(message, source, lineNumber);
}

void EAssertionFailed::Assertion(bool assertion, const char * source, long lineNumber)
{
	if (!assertion)
		throw new EAssertionFailed("Assertion failed", source, lineNumber);
}


// --- Class EDeviceError ---

EDeviceError::EDeviceError(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("EDeviceError", message, source, lineNumber)
{
}

#ifdef _WINDOWS
	EDeviceError * EDeviceError::CreateDeviceError(const char * message, HRESULT result, const char * source, long lineNumber)
	{
		return new EDeviceError(String(message) + "\nCause: " + GetErrorMessage(result), source, lineNumber);
	}

	String EDeviceError::GetErrorMessage(HRESULT result)
	{
		unsigned long facilityId = (((unsigned long) result) & 0x07FF0000UL) >> 16;
		unsigned long errorCode = ((unsigned long) result) & 0x0000FFFFUL;

		String message("Error ");
		message += errorCode;
		message += " in facility ";
		message += facilityId;

		char systemMessage[256];
		systemMessage[0] = 0;
		::FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, (unsigned long) result,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
			systemMessage, 255, NULL);

		#if defined(D3D_OVERLOADS)
			if (!systemMessage[0] && facilityId == (_FACDD & 0x07FF))
			{
				const char * directXMessage = DXGetErrorString9(result);
				if (directXMessage != NULL)
				{
					strncpy(systemMessage, directXMessage, 255);
					systemMessage[255] = 0;
				}
			}
		#endif

		if (systemMessage[0])
		{
			message += ": ";
			message += systemMessage;
			message.TrimRight();
		}

		return message;
	}
#endif


// --- Class EIllegalArgument ---

EIllegalArgument::EIllegalArgument(const char * message, const char * source,
	long lineNumber)
	: EAutoBreaking("EIllegalArgument", message, source, lineNumber)
{
}


// --- Class EIllegalState ---

EIllegalState::EIllegalState(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("EIllegalState", message, source, lineNumber)
{
}


// --- Class EIndexOutOfBounds ---

EIndexOutOfBounds::EIndexOutOfBounds(const char * message, const char * source,
	long lineNumber)
	: EAutoBreaking("EIndexOutOfBounds", message, source, lineNumber)
{
}


// --- Class EInternalError ---

EInternalError::EInternalError(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("EInternalError", message, source, lineNumber)
{
}


// --- Class EIOError ---

EIOError::EIOError(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("EIOError", message, source, lineNumber)
{
}


// --- Class ENoSuchElement ---

ENoSuchElement::ENoSuchElement(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("ENoSuchElement", message, source, lineNumber)
{
}


// --- Class ENotSupported ---

ENotSupported::ENotSupported(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("ENotSupported", message, source, lineNumber)
{
}


// --- Class ENullPointer ---

ENullPointer::ENullPointer(const char * message, const char * source, long lineNumber)
	: EAutoBreaking("ENullPointer", message, source, lineNumber)
{
}

void ENullPointer::AssertionValid(const void * pointer, const char * message,
	const char * source, long lineNumber)
{
	if (pointer == NULL)
		throw new ENullPointer(message, source, lineNumber);
}

void ENullPointer::AssertionValid(const void * pointer, const char * source, long lineNumber)
{
	if (pointer == NULL)
		throw new ENullPointer("Pointer is null", source, lineNumber);
}


// --- Class EOutOfMemory ---

EOutOfMemory::EOutOfMemory()
	: Exception("EOutOfMemory", "Insufficient free memory", "No info", 0)
{
}

EOutOfMemory EOutOfMemory::Singleton;
