/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_STACK_H
#define __TOOLBOX_STACK_H


namespace toolbox
{
	/**
	 * A ContainingStack contains a variable number
	 * of items, that are stacked on top of each other
	 * and are only accessable one by one from the top
	 * end of the stack. The chaining makes enqueueing
	 * and dequeueing fast.
	 * @note ContainingStacks contain ContainingStackItems,
	 * that are a little more performant, but may not
	 * be added twice to the same stack, nor be added
	 * to two different stacks. If you require this,
	 * use the PointeredStack and PointeredStackItem.
	 * To use ContainingStacks, derive at least from the
	 * ContainingStackItem, and put your data in there.
	 * @see See collections.txt for an overview of the
	 * available collection classes.
	 * @see PointeredStack
	 * @see PointeredStackItem
	 * @author Thomas Jacob
	 */
	template <class ITEMTYPE> class ContainingStack
	{
		private:

			/**
			 * The number of items in the stack.
			 */
			long Count;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * The top stack item.
			 */
			ITEMTYPE * Top;

		public:

			/**
			 * Creates a new stack.
			 */
			ContainingStack();

			/**
			 * Destroys the stack. When deleting the stack,
			 * all contained items are also freed.
			 */
			virtual ~ContainingStack();

			/**
			 * Deletes the top item from the stack.
			 * The item is removed from the stack and freed.
			 * @return If the deletion was successful.
			 */
			bool Delete();

			/**
			 * Deletes all items from the stack.
			 * All items are removed from the stack and freed.
			 */
			void DeleteAll();

			/**
			 * Returns the number of items in the stack.
			 * @return The number of items in the stack.
			 */
			inline long GetCount() const;

			/**
			 * Returns, if there are no items in the stack.
			 */
			inline bool IsEmpty() const;

			/**
			 * Returns the top item of the stack.
			 * @return The top item of the stack.
			 */
			inline ITEMTYPE * Peek() const;

			/**
			 * Removes the top item of the stack. The item is
			 * not freed, but returned.
			 * @return The item.
			 */
			ITEMTYPE * Pop();

			/**
			 * Removes all items from the stack
			 * without freeing them or returning them.
			 * @warning No item is freed, so you have to have
			 * a pointer to the objects, or the memory is lost.
			 */
			void PopAll();

			/**
			 * Puts an item on the top of the stack.
			 * @param Item The item to be put on top.
			 */
			void Push(ITEMTYPE * NewItem);

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A ContainingStackItem is one item stored in
	 * the ContainingStack. It may not be added twice
	 * to the same stack, nor be added to two different
	 * stacks, since the pointering technique is
	 * designed for performance, not for functionality.
	 * If you require this, use the PointeredStack and
	 * the PointeredStackItem.
	 * @see PointeredStack
	 * @see PointeredStackItem
	 * @author Thomas Jacob
	 */
	template <class ITEMTYPE> class ContainingStackItem
	{
		friend class ContainingStack<ITEMTYPE>;

		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

			/**
			 * A pointer to the next ContainingStackItem
			 * in the stack chain.
			 */
			ITEMTYPE * NextItem;

		public:

			/**
			 * Creates a new ContainingStackItem.
			 */
			ContainingStackItem();

			/**
			 * Destroys the ContainingStackItem.
			 */
			virtual ~ContainingStackItem();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A PointeredStack contains a variable number
	 * of items, that are stacked on top of each other
	 * and are only accessable one by one from the top
	 * end of the stack. The chaining makes enqueueing
	 * and dequeueing fast.
	 * @note PointeredStacks contain PointeredStackItems,
	 * that are a little less performant, but may be
	 * added twice to the same stack, and be added
	 * to two different stacks. If do not require this,
	 * use the ContainingStack and ContainingStackItem.
	 * To use PointeredStacks, store your data in the data
	 * field of the PointeredStackItem.
	 * @see ContainingStack
	 * @see ContainingStackItem
	 * @author Thomas Jacob
	 */
	class PointeredStack : public ContainingStack<PointeredStackItem>
	{
		private:

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new PointeredStack.
			 */
			PointeredStack();

			/**
			 * Destroys the PointeredStack. When deleting the stack,
			 * all contained items are also freed. Since the data
			 * is only pointered to and not held in the items,
			 * this has no effect to the data in the items.
			 */
			virtual ~PointeredStack();

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif
	};


	/**
	 * A PointeredStackItem is one item stored in
	 * the PointeredStack. It may be added twice
	 * to the same stack and be added to two different
	 * stacks, since the pointering technique is designed
	 * for this. This results in a little less performance,
	 * and if you require a very good one, use the
	 * ContainingStack and the ContainingStackItem instead.
	 * @see ContainingStack
	 * @see ContainingStackItem
	 * @author Thomas Jacob
	 */
	class PointeredStackItem : public ContainingStackItem<PointeredStackItem>
	{
		private:

			/**
			 * The application's data to be stored in the item,
			 */
			void * Data;

			#ifdef _TOOLBOX_TEST
				/**
				 * The number of existing instances for testing purposes.
				 */
				static int InstanceCount;
			#endif

		public:

			/**
			 * Creates a new PointeredStackItem.
			 */
			PointeredStackItem();

			/**
			 * Destroys the PointeredStackItem.
			 */
			virtual ~PointeredStackItem();

			/**
			 * Returns the stored data of the item.
			 * @return The stored data of the item.
			 */
			inline void * GetData() const;

			#ifdef _TOOLBOX_TEST
				/**
				 * Performs a series of tests to check if the class
				 * runs properly. Produces standard output on errors.
				 * @param performedTests Is increased by one for each performed test.
				 * @param failedTests Is increased by one for each failed test.
				 */
				static void RunTestSuite(int * performedTests, int * failedTests);
			#endif

			/**
			 * Sets the data to be stored in the item.
			 * @param Data The new data to be stored.
			 */
			inline void SetData(void * Data);
	};
}


#endif
